#!/usr/bin/perl
#####################################################################
# Function: common.pm
#
# 概要:
# PG-REX 便利ツールから呼び出すモジュールの集まり
# 
# 特記事項:
# なし
#
# Copyright (c) 2012-2014, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
#
#####################################################################
use warnings;
use strict;
use Switch;
use Net::OpenSSH;
use include::command;
use Class::Struct;

BEGIN {
    if ($ENV{"LANG"} =~ m/ja/i){
        eval qq{
            use po::ja;
        };
    }
    else{
        eval qq{
            use po::en;
        };
    }
};

struct Ssh_info => {
	address => '$',
	user => '$',
	pass => '$'
};

sub postgres_running{
    my ($pgdata, $ssh) = @_;
    my $result;
	my @tmp_result;

	if ($ssh) {
		@tmp_result = ssh_exec_command($ssh->address(), $ssh->user(), $ssh->pass(), "$PS aux | $GREP \"bin/postgres -D $pgdata \" | $GREP -v \"grep\"");
		$result = $tmp_result[0];
	}
	else {
    	$result = `$PS aux | $GREP "bin/postgres -D $pgdata " | $GREP -v "grep"`;
	}

    # PostgreSQL のプロセスを発見したら1を返却する
    if ($result ne ""){
        return 1;
    }
    return 0;
}


sub pacemaker_running{
	my ($ssh) = @_;
    my $result;
	my @tmp_result;

    if ($ssh) {
		@tmp_result = ssh_exec_command($ssh->address(), $ssh->user(), $ssh->pass(), "$PS aux | $GREP \"heartbeat: master control process\" | $GREP -v \"grep\"");
		$result = $tmp_result[0];
	}
	else {
		$result = `$PS aux | $GREP "heartbeat: master control process" | $GREP -v "grep"`;
	}

    # Pacemaker のプロセスを発見したら1を返却する
    if ($result ne ""){
        return 1;
    }
    return 0;
}


sub pacemaker_online{
    my ($my_node) = @_;
    my @results;
    my @crm_results;
    my $exit_code;

    $results[0] = `$CRM_MON -fA -1`;
    $results[1] = $? >> 8;

    $exit_code = $results[1];
    # Pacemaker が起動している場合は $exit_code = 0 、起動していない場合は $exit_code = 10
    if ($exit_code != 0 && $exit_code != 10){
        printlog("ERROR", COMMON_MS0001);
    }

    # crm_mon の結果を行ごとの配列に格納する
    @crm_results = split (/\n/, $results[0]);

    # Pacemaker が Online だったら1を返却する
    foreach my $line (@crm_results){
        # crm_mon の結果に「Online: [ ～ <マシンのホスト名>」の行が存在すると OK
        if ($line =~ /Online\:\s+\[(\s+\S+)?\s+$my_node\s+/){
            return 1;
        }
    }
    return 0;

}


sub pgrex_failed_action{
    my ($my_node, $pg_primitive_resource_id) = @_;
    my @results;
    my @crm_results;
    my $exit_code;

    $results[0] = `$CRM_MON -fA -1`;
    $results[1] = $? >> 8;

    $exit_code = $results[1];
    # Pacemaker が起動している場合は $exit_code = 0 、起動していない場合は $exit_code = 10
    if ($exit_code != 0 && $exit_code != 10){
        printlog("ERROR", COMMON_MS0001);
    }
    
    # crm_mon の結果を行ごとの配列に格納する
    @crm_results = split (/\n/, $results[0]);

    my $line_count = 0;
    my $line_tmp = 0;
    # crm_mon の結果に failed Action の内容が存在していたら1を返却する
    foreach my $line (@crm_results){
        $line_count++;

        # crm_mon の結果に「Failed actions:」の行が存在した後、
        # 1行後に「<PostgreSQLのPrimitiveのリソースID>:(.) ～ node=<マシンのホスト名>」の行が存在すると OK
        # ※(.) = 文字の種類は問わず一文字
        if ($line =~ /Failed\s+actions:/){
            $line_tmp = $line_count;
        }
        if ($line =~ /$pg_primitive_resource_id:[0-1]+_\S+_[0-9]+\s+\(node=$my_node/ && $line_count == $line_tmp + 1){
            return 1;
        }
    }
    return 0;

}


sub network_running{
    my ($my_node, $another_node, @nic_list) = @_;
    my $result;
    my $attribute;
    my $exit_code;

    # コマンド
    # 「crm_attribute -t status -N <ノード名> --name <ノード名>-<NIC名> -G -q」
    # の結果が2ノードとも"up"であると OK
    foreach my $nic (@nic_list){
        $attribute = $another_node."-".$nic;
        $result = `$CRM_ATTRIBUTE -t status -N $my_node --name $attribute -G -q 2> /dev/null`;
        chomp $result;
        if ($result ne "up"){
            return 0;
        }
    }
    foreach my $nic (@nic_list){
        $attribute = $my_node."-".$nic;
        $result = `$CRM_ATTRIBUTE -t status -N $another_node --name $attribute -G -q 2> /dev/null`;
        chomp $result;
        if ($result ne "up"){
            return 0;
        }
    }
    return 1;

}


sub master_running{
    my ($my_node, $master_resource_id, $pg_primitive_resource_id, $ssh_pass, $address_connect) = @_;
    my @results;
    my $command;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <MasterリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource <MasterリソースID> is running on: <マシンのホスト名> Master"である
    #   *  コマンド
    #     「crm_attribute -l forever -N <マシンのホスト名> --name <PostgreSQLのPrimitiveのリソースID>-data-status -G -q」
    #      の結果が"LATEST"である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name <PostgreSQLのPrimitiveのリソースID>-status -G -q」
    #      の結果が"PRI"である
    ##

    $command = "$CRM_RESOURCE -r $master_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "resource $master_resource_id is running on: $my_node Master"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -l forever -N $my_node --name ${pg_primitive_resource_id}-data-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "LATEST"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name ${pg_primitive_resource_id}-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "PRI"){
        return 0;
    }

    return 1;
}


sub slave_running{
    my ($my_node, $master_resource_id, $pg_primitive_resource_id, $ssh_pass, $address_connect) = @_;
    my @results;
    my $command;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <MasterリソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource <MasterリソースID> is running on: <マシンのホスト名> "である
    #   *  コマンド
    #     「crm_attribute -l forever -N <マシンのホスト名> --name <PostgreSQLのPrimitiveのリソースID>-data-status -G -q」
    #      の結果が"STREAMING|SYNC"である
    #   *  コマンド
    #     「crm_attribute -t status -N <マシンのホスト名> --name <PostgreSQLのPrimitiveのリソースID>-status -G -q」
    #      の結果が"HS:SYNC"である
    ##

    $command = "$CRM_RESOURCE -r $master_resource_id -W 2> /dev/null | $GREP \" $my_node \"";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "resource $master_resource_id is running on: $my_node "){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -l forever -N $my_node --name ${pg_primitive_resource_id}-data-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "STREAMING|SYNC"){
        return 0;
    }

    $command = "$CRM_ATTRIBUTE -t status -N $my_node --name ${pg_primitive_resource_id}-status -G -q 2> /dev/null";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        chomp $results[0];
    }
    if ($results[0] ne "HS:sync"){
        return 0;
    }

    return 1;

}


sub vip_running{
    my ($my_node, $resource_id) = @_;
    my $result;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID> -W 2> /dev/null」
    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
    ##

    $result = `$CRM_RESOURCE -r $resource_id -W 2> /dev/null`;
    chomp $result;
    if ($result ne "resource $resource_id is running on: $my_node "){
        return 0;
    }

    return 1;

}


sub stonith_running{
    my ($my_node, $resource_id) = @_;
    my $result;

    ##
    # 以下の条件を満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID_1> -W 2> /dev/null」
    #     「crm_resource -r <リソースID_2> -W 2> /dev/null」
    #      のどちらも実行して、どちらかの結果が
    #      "resource <指定したリソースID> is running on: <マシンのホスト名> "である
    ##

    $result = `$CRM_RESOURCE -r $resource_id->[0] -W 2> /dev/null`;
    chomp $result;
    if ($result eq "resource $resource_id->[0] is running on: $my_node "){
        return 1;
    }

    $result = `$CRM_RESOURCE -r $resource_id->[1] -W 2> /dev/null`;
    chomp $result;
    if ($result eq "resource $resource_id->[1] is running on: $my_node "){
        return 1;
    }

    return 0;

}


sub pingd_running{
    my ($my_node, $resource_id) = @_;
    my $result;
    my @resource_id;
    my $array_num;
    my $resource_check_count = 0;

    ##
    # 以下の条件を指定されたリソースの数だけ満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
    ##

    @resource_id = split(/\s*,\s*/, $resource_id);
    $array_num = scalar(@resource_id);
    
    foreach my $id (@resource_id){
        $result = `$CRM_RESOURCE -r $id -W 2> /dev/null | $GREP \" $my_node \"`;
        chomp $result;
        if ($result eq "resource $id is running on: $my_node "){
            $resource_check_count ++;
        }
    }

    if ($resource_check_count == $array_num){
        return 1;
    }
    
    return 0;

}


sub diskd_running{
    my ($my_node, $resource_id) = @_;
    my $result;
    my @resource_id;
    my $array_num;
    my $resource_check_count = 0;

    ##
    # 以下の条件を指定されたリソースの数だけ満たしたとき結果を OK とする
    #   *  コマンド
    #     「crm_resource -r <リソースID> -W 2> /dev/null | grep " <マシンのホスト名> "」
    #      の結果が"resource <リソースID> is running on: <マシンのホスト名> "である
    ##

    @resource_id = split(/\s*,\s*/, $resource_id);
    $array_num = scalar(@resource_id);
    
    foreach my $id (@resource_id){
        $result = `$CRM_RESOURCE -r $id -W 2> /dev/null | $GREP \" $my_node \"`;
        chomp $result;
        if ($result eq "resource $id is running on: $my_node "){
            $resource_check_count ++;
        }
    }

    if ($resource_check_count == $array_num){
        return 1;
    }
    
    return 0;

}


sub read_cib{
    my ($cib_path, $psql_path, $pg_primitive_resource_id, $kill_when_no_data, $config_path, $dbcluster_dir) = @_;
    my @cib_strings;
    my %cib_value;
    my $my_node;
    my $my_node_id;
    my $my_pgsql_data_status_id;
    my @check_key_list = ("pgdata", "master_ip", "repuser");
    my $config_comment = "# この設定値はツールが自動で追加したものです。設定行が複数行出る場合がありますが動作に影響はありません。";
    my $result;

    $my_node = exec_command("$UNAME -n");
    chomp $my_node;

    # PG-REX のバージョンを確認し、その結果に合わせて tmpdir のデフォルト値を設定する
    $result = get_pg_version($psql_path);
    if ($result =~ /^9\.1[^0-9]/){
        $cib_value{"tmpdir"} = "/var/lib/pgsql";
    }
    else{
        $cib_value{"tmpdir"} = "/var/lib/pgsql/tmp";
    }

    open (FILE, $cib_path) or printlog("ERROR", COMMON_MS0023, $cib_path);
    @cib_strings = <FILE>;
    close (FILE);

    # PostgreSQL のリソース部分からパラメータ値を取得
    foreach my $line (@cib_strings){
        # <PostgreSQLのPrimitiveのリソースID>-instance_attributes format : 
        #     <nvpair id="<PostgreSQLのPrimitiveのリソースID>-instance_attributes-<文字列>" name="<name>" value="<value>"/>
        if ($line =~ /\"$pg_primitive_resource_id-instance_attributes-\S+\"\s+name=\"([^\"\s]+)\"\s+value=\"([^\"]+)\"/){
            $cib_value{$1} = $2;
        }
        # node id format : 
        #     <node (id="<node id>" type="<type>" uname="<node name>")>
        #     ※()の中のタグの順番は不定
        if ($line =~ /uname=\"$my_node\"/ && $line =~ /id=\"([^\"\s]+)\"/){
            $my_node_id = $1;
            $my_pgsql_data_status_id = "nodes-".$my_node_id."-".$pg_primitive_resource_id."-data-status";
        }
    }

    foreach my $line (@cib_strings){
        if ($my_pgsql_data_status_id && $line =~ /id=\"$my_pgsql_data_status_id\"/ && $line =~ /value=\"([^\"\s]+)\"/){
            $cib_value{"pgsql_data_status"} = $1;
        }
    }
    
    foreach my $key (@check_key_list){
        if (!exists($cib_value{$key}) && !$kill_when_no_data){
            printlog("ERROR", COMMON_MS0003, $pg_primitive_resource_id, $key);
        }
    }

    # pg-rex_tools.conf にDBクラスタパスの記述が無い場合追加する
    if (! defined ($dbcluster_dir)){
        exec_command("$ECHO \"ARCHIVE_DELETE_DBCLUSTER_DIR = $cib_value{'pgdata'}  $config_comment\" >> $config_path");
    
    }
    return %cib_value;
}


sub read_config{
    my ($config_path) = @_;
    my @config_strings;
    my %config_value;
    my @check_key_list = ("Archive_dir", "D-LAN_IPAddress", 
                         "PG-REX_Master_ResourceID","PG-REX_Primitive_ResourceID");
    my $result;

    $config_value{'STONITH'} = "enable";
    $config_value{'VIP_SLAVE'} = "enable";
    $config_value{'PEER_NODE_SSH_PASS_MODE'} = "manual";
    $config_value{'BACKUP_NODE_SSH_PASS_MODE'} = "manual";
    
    open (FILE, $config_path) or printlog("ERROR", COMMON_MS0024, $config_path);
    @config_strings = <FILE>;
    close (FILE);

    foreach my $line (@config_strings){
        # コメントの削除
        # comment format : #<comment>
        $line =~ s/\#.*//g;

        # 行末の空白の削除
        # space format : <space><\n>
        $line =~ s/\s+$//g;

        # 全角の空白を半角空白に変換
        # two-byte characterspace format : <two-byte character space>
        $line =~ s/　/ /g;

        # environment setting value format : <<value name> = <parameter>> or <<value name> = <parameter>,<parameter>>
        if ($line =~ /^\s*(\S+)\s*=\s*(.+)$/){
            $config_value{$1} = $2;
        }
    }

    foreach my $key (@check_key_list) {
        if (!exists($config_value{$key})){
            printlog("ERROR", COMMON_MS0004, $key);
        }
    }
    if ($config_value{'Archive_dir'} !~ /^\//){
        printlog("ERROR", COMMON_MS0005, "Archive_dir");
    }
    if ($config_value{'STONITH'} ne "enable" && $config_value{'STONITH'} ne "disable"){
        printlog("ERROR", COMMON_MS0006);
    }
    if ($config_value{'VIP_SLAVE'} ne "enable" && $config_value{'VIP_SLAVE'} ne "disable"){
        printlog("ERROR", COMMON_MS0021);
    }
    if ($config_value{'PEER_NODE_SSH_PASS_MODE'} ne "manual" && $config_value{'PEER_NODE_SSH_PASS_MODE'} ne "passfile"
        && $config_value{'PEER_NODE_SSH_PASS_MODE'} ne "nopass" ){
       printlog("ERROR", COMMON_MS0034, "PEER_NODE_SSH_PASS_MODE");
    }
    if ($config_value{'BACKUP_NODE_SSH_PASS_MODE'} ne "manual" && $config_value{'BACKUP_NODE_SSH_PASS_MODE'} ne "passfile"
        && $config_value{'BACKUP_NODE_SSH_PASS_MODE'} ne "nopass" ){
       printlog("ERROR", COMMON_MS0034, "BACKUP_NODE_SSH_PASS_MODE");
    }
    if ($config_value{'PEER_NODE_SSH_PASS_MODE'} eq "passfile" && $config_value{'PEER_NODE_SSH_PASS_FILE'} !~ /^\//){
        printlog("ERROR", COMMON_MS0005, "PEER_NODE_SSH_PASS_FILE");
    }
    if ($config_value{'BACKUP_NODE_SSH_PASS_MODE'} eq "passfile" && $config_value{'BACKUP_NODE_SSH_PASS_FILE'} !~ /^\//){
        printlog("ERROR", COMMON_MS0005, "BACKUP_NODE_SSH_PASS_FILE");
    }

    # VIP_SLAVE が disable の場合、VIP-SLAVE_ResourceID の値を空にする
    if ($config_value{'VIP_SLAVE'} eq "disable"){
        $config_value{"VIP-SLAVE_ResourceID"} = "";
    }

    # STONITH が enable の場合、 Group のリソース ID を配列に分解しハッシュに格納する
    # また、disable の場合、STONITH_ResourceID の値を空にする
    if ($config_value{'STONITH'} eq "enable" && exists($config_value{"STONITH_ResourceID"})){
        my @stonith_resources = split(/\s*,\s*/, $config_value{"STONITH_ResourceID"});
        if (scalar(@stonith_resources) != 2){
            printlog("ERROR", COMMON_MS0022);
        }
        $config_value{"STONITH_ResourceID"} = [$stonith_resources[0], $stonith_resources[1]];
    }
    else{
        $config_value{"STONITH_ResourceID"} = "";
    }

    my @ifconfig_ip = ();
    # ","で区切られた D-LAN の IP アドレスをそれぞれ配列に格納
    my @d_lan_ip = split(/\s*,\s*/, $config_value{"D-LAN_IPAddress"});
    if (scalar(@d_lan_ip) != 2){
        printlog("ERROR", COMMON_MS0007);
    }

    # コマンド実行マシンに設定してある IP アドレスを配列に格納
    $result = `$IFCONFIG | $GREP "inet addr"`;
    my @ifconfig_strings = split(/\n/, $result);
    foreach my $line (@ifconfig_strings){
        # IP address format : <inet addr:<value>>
        if ($line =~ /inet\saddr:([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/) {
            push (@ifconfig_ip , $1);
        }
    }
    if (!scalar(@ifconfig_ip)){
        printlog("ERROR", COMMON_MS0008);
    }

    foreach my $line (@d_lan_ip){
        if (grep {$_ eq $line} @ifconfig_ip){
            $config_value{"My_D-LAN_IPAddress"} = $line;
        }
        else {
            $config_value{"Another_D-LAN_IPAddress"} = $line;
        }
    }

    if (!exists($config_value{'My_D-LAN_IPAddress'})){
        printlog("ERROR", COMMON_MS0009);
    }
    if (!exists($config_value{'Another_D-LAN_IPAddress'})){
        printlog("ERROR", COMMON_MS0010);
    }

    return %config_value;
}


sub read_hacf{
    my ($hacf_path, $status) = @_;
    my @hacf_strings;
    my %hacf_value;
    my @ic_dev;
    my @node;
    my $check;

    open (FILE, $hacf_path) or printlog("ERROR", COMMON_MS0025, $hacf_path);
    @hacf_strings = <FILE>;
    close (FILE);

    # IC-LAN に使用しているデバイス名とノード名を取得
    foreach my $line (@hacf_strings){
        chomp $line;
        # comment format : #<comment>
        $line =~ s/\#.*//g;
        # IC-LAN device name format : bcast <device name> or ucast <device name> <peer-ip-addr> or 
        #                             mcast(mcast6 ) <device name> <multicast group> <portnumber> <ttl> <loop>
        if ($line =~ /^\s*(bcast|ucast|mcast|mcast6)\s+(\S+)/){
            push(@ic_dev, $2);
        }
        # node name format : node <node name>
        if ($line =~ /^\s*node\s+(\S+)/){
            push(@node, $1);
        }
    }
    
    # 起動処理の時のみIC-LAN に使用しているデバイスの数を確認する
    if ($status && scalar(@ic_dev) == 0){
        printlog("ERROR", COMMON_MS0019);
    }
    if ($status && scalar(@ic_dev) == 1){
        printlog("LOG", COMMON_MS0027);
    }

    # ノードの数を確認する
    if (scalar(@node) != 2){
        printlog("ERROR", COMMON_MS0020);
    }

    %hacf_value = (ic_dev => [@ic_dev],
                   node   => [$node[0], $node[1]]);

    return %hacf_value;
}


sub exec_command {
    my ($command) = @_;

    my $return = `$command`;
    my $exit_code = $? >> 8;
    if ($exit_code != 0){
        printlog("ERROR", COMMON_MS0011, $command);
    }

    return $return;
}


sub ssh_exec_command {
    my ($address, $user, $pass, $command) = @_;
    my @results;
    my $ssh;

    if ($pass eq ""){
        $ssh = Net::OpenSSH->new($address, user => $user);
    }
    else{
        $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    }
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    $results[0] = $ssh->capture($command);
    $results[1] = $? >> 8;
    chomp $results[0];

    return @results;
}


sub rsync_exec_command {
    my ($address, $user, $pass, $remote, $local, $put_flag) = @_; 
    my $ssh;

    if ($pass eq ""){
        $ssh = Net::OpenSSH->new($address, user => $user);
    }
    else{
        $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    }
    $ssh->error and printlog("ERROR", COMMON_MS0016);

    my %opts = (
        archive => 1,
        verbose => 1,
    );

	if ($put_flag){
		$ssh-> rsync_put(\%opts, $local, $remote) or $ssh->error and printlog("ERROR", COMMON_MS0017);
	}
	else {
    	$ssh-> rsync_get(\%opts, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0017);
    }

    return 0;
}


sub scp_exec_command {
    my ($address, $user, $pass, $remote, $local) = @_; 
    my $ssh;

    if ($pass eq ""){
        $ssh = Net::OpenSSH->new($address, user => $user);
    }
    else{
        $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    }
    $ssh->error and printlog("ERROR", COMMON_MS0016);

    $ssh-> scp_get({copy_attrs => 1}, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0018);

    return 0;
}


sub get_xlogfile_name{
    my ($location, $this_time_line_id) = @_;
    my $uxlog_id;
    my $uxrec_off;
    my $xlog_id;
    my $xlog_seg;
    my $xlog_file_name;
    
    # XLOG 位置の情報をパース
    # location format : <xlogid>/<xrecoff>
    # <xlogid>は1,2桁の16進数
    # <xrecoff>は7.8桁の16進数
    if ($location !~ /^([0-9A-F]+)\/([0-9A-F]{7,8})$/){
        printlog("ERROR", COMMON_MS0013);
    }
    $uxlog_id = $1;
    $uxrec_off = $2;

    # 一つ前のセグメントを取得
    # $xlog_segは、上で取得した16進数の値から1を引いて5桁以下をシフトした値になる
    $xlog_id = hex($uxlog_id);
    my $uxrec_off_tmp = hex($uxrec_off);
    $xlog_seg = sprintf("%d", ($uxrec_off_tmp - 1)/(16 ** 6));

    # 一つ前のセグメントの WAL ファイル名を取得
    $xlog_file_name = sprintf("%08X%08X%08X", $this_time_line_id, $xlog_id, $xlog_seg);

    return $xlog_file_name;
}


sub get_controldata_value{
    my (@controldata_strings) = @_;
    my %controldata_value;
    my @key_list = ("update_time", "database", "time_line_id", "xlog_location");

    foreach my $line (@controldata_strings){
        # pg_control last modified format : pg_control last modified: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
        if ($line =~ /^pg_control\s+last\s+modified:\s+(.+)$/){
            $controldata_value{'update_time'} = $1;
        }
        # Database system identifier format : Database system identifier: <Database>
        if ($line =~ /^Database\s+system\s+identifier:\s+([0-9A-F]+)$/){
            $controldata_value{'database'} = $1;
        }
        # TimeLineID format : Latest checkpoint's TimeLineID: <TimeLineID>
        # <TimeLineID>は10進数
        if ($line =~ /^Latest\s+checkpoint's\s+TimeLineID:\s+([0-9A-F]+)$/){
            $controldata_value{'time_line_id'} = $1;
        }
        # location format : Latest checkpoint's REDO location: <location>
        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+location:\s+([0-9A-F\/]+)$/){
            $controldata_value{'xlog_location'} = $1;
        }
    }

    foreach my $key (@key_list) {
        if (!exists($controldata_value{$key})){
            printlog("ERROR", COMMON_MS0014);
        }
    }


    return %controldata_value;
}


sub get_recoverywal {
    my (@controldata_strings) = @_;
    my $wal = '';

    my %controldata_value = get_controldata_value(@controldata_strings);

    $wal = get_xlogfile_name($controldata_value{'xlog_location'}, $controldata_value{'time_line_id'});

    return $wal;
}


sub get_node{
    my ($another_address, $ssh_pass, $exec_user) = @_;
    my @results;
    my %node_value = ("my_node"      => '',
                      "another_node" => '');

    $node_value{'my_node'} = exec_command("$UNAME -n");
    chomp $node_value{'my_node'};

    if (!$exec_user){
        $exec_user = "root";
    }
    
    @results = ssh_exec_command($another_address, $exec_user, $ssh_pass, "$UNAME -n");
    $node_value{'another_node'} = $results[0];
    chomp $node_value{'another_node'};

    if (!$node_value{'another_node'}){
        printlog("ERROR", COMMON_MS0015);
    }

    return %node_value;
}


sub get_pg_version{
    my ($psql_path) = @_;
    my $result;
    my @results;
    my $pg_command_user = "postgres";

    $result = exec_command("$SU - $pg_command_user -c \"$psql_path --version \"");

    @results = split(/\s/,$result);
    chomp $results[2];

    return $results[2];
}

sub is_dbcluster_check_support{
    my ($pg_version) = @_;
    my @version_factor;

    @version_factor = split(/\./, $pg_version);

    if (($version_factor[0] >= 10) ||
        ($version_factor[0] == 9 && $version_factor[1] >= 3)){
        return 0;
    }

    return 1;
}

sub is_switchover_support{
    my ($pg_version) = @_;
    my @version_factor;

    @version_factor = split(/\./, $pg_version);

    if (($version_factor[0] == 9 && $version_factor[1] == 1 && $version_factor[2] >= 10) ||
        ($version_factor[0] == 9 && $version_factor[1] == 2 && $version_factor[2] >= 5) ||
        ($version_factor[0] == 9 && $version_factor[1] >= 3)){
        return 1;
    }

    return 0;
}

sub get_pg_command_path {
    my ($path, $exec_user) = @_;
    my %command_path;
    my @command_list = ("psql", "pg_controldata", "pg_basebackup", "vacuumdb");
    my $pg_command_user = "postgres";

    if (!$path){
        $path = "";
    }
    if ($path && $path =~ /\S+\/$/){
        $path =~ s/\/$//;
    }

    foreach my $command (@command_list){
        if (!defined($exec_user) || $exec_user eq "root"){
            $command_path{$command} = `$SU - $pg_command_user -c "$WHICH $command" 2> /dev/null`;
        }
        else{
            $command_path{$command} = `$WHICH $command 2> /dev/null`;
        }
        if (!$command_path{$command}){
            $command_path{$command} = $path."/".$command;

            # psqlの場合のみPGPATHが正しく設定されているかを確認
            if ($command eq "psql"){
                `$WHICH $command_path{$command} 2> /dev/null`;
                my $return = $? >> 8;
                if ($return != 0){
                    printlog("ERROR", COMMON_MS0036, $path);
                }
            }   
        }
        else{
            chomp $command_path{$command};
        }
    }

    return %command_path;
}

sub get_pm_version{
    my $result;
    my @results;

    $result = exec_command("$CRM_MON --version");

    @results = split(/\s/,$result);
    chomp $results[1];

    return $results[1];
}

sub check_user{
    my $result;

    $result = exec_command("$WHOAMI");
    chomp $result;
    if ($result ne "root"){
        printlog("ERROR", COMMON_MS0026, $result);
    }
}

sub printlog {
    my ($log_level, $message, @values) = @_;

    for (my $count=0; $count < scalar(@values); $count++){
        $message =~ s/\[$count\]/$values[$count]/;
    }

    if ($log_level eq "ERROR"){
        die $message;
    }
    else{
        print $message;
    }

}


sub get_ssh_passwd {
    # PG-REX相手ノードへの ssh 接続の為の情報を取得
    my ($another_ip, $ssh_pass_mode, $ssh_pass_file) = @_;
    my $exec_user;
    my $passwd;
    my @file_stat;
    my $file_permission;

    $exec_user = exec_command("$WHOAMI");
    chomp $exec_user;

    if ($ssh_pass_mode eq "manual"){
        printlog("LOG", COMMON_MS0029, $exec_user, $another_ip );
        `$STTY -echo`;
        $passwd = <STDIN>;
        chomp $passwd;
        `$STTY echo`;
        printlog("LOG", COMMON_MS0030);
    }
    elsif ($ssh_pass_mode eq "nopass"){
        $passwd = "";
    }
    else {
        if (! -f $ssh_pass_file){
            printlog("ERROR", COMMON_MS0031, $ssh_pass_file);
        }
        # パスワードファイルの権限が600に設定されているかの確認
        @file_stat = stat($ssh_pass_file);
        $file_permission = substr((sprintf "%03o", $file_stat[2]), -3);
        if ($file_permission ne "600"){
             printlog("ERROR", COMMON_MS0035, $ssh_pass_file);
        }

        open (FILE, $ssh_pass_file) or printlog("ERROR", COMMON_MS0032, $ssh_pass_file);
        $passwd = <FILE>;
        close (FILE);

        chomp $passwd;

        if ( $passwd eq "" ){
            printlog("ERROR", COMMON_MS0033);
        }
    }

    return $passwd;
}

1;
